@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: cli.py 838 2007-12-20 01:42:31Z pronovic $
+# Revision : $Id: cli.py 935 2009-03-29 18:56:50Z pronovic $
 # Purpose  : Provides command-line interface implementation.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -89,7 +89,7 @@ import getopt
 from CedarBackup2.release import AUTHOR, EMAIL, VERSION, DATE, COPYRIGHT
 from CedarBackup2.util import RestrictedContentList, DirectedGraph, PathResolverSingleton
 from CedarBackup2.util import sortDict, splitCommandLine, executeCommand, getFunctionReference
-from CedarBackup2.util import getUidGid, encodePath
+from CedarBackup2.util import getUidGid, encodePath, Diagnostics
 from CedarBackup2.config import Config
 from CedarBackup2.peer import RemotePeer
 from CedarBackup2.actions.collect import executeCollect
@@ -130,11 +130,11 @@ VALID_ACTIONS      = [ "collect", "stage", "store", "purge", "rebuild", "validat
 COMBINE_ACTIONS    = [ "collect", "stage", "store", "purge", ]
 NONCOMBINE_ACTIONS = [ "rebuild", "validate", "initialize", "all", ]
 
-SHORT_SWITCHES     = "hVbqc:fMNl:o:m:Ods"
+SHORT_SWITCHES     = "hVbqc:fMNl:o:m:OdsD"
 LONG_SWITCHES      = [ 'help', 'version', 'verbose', 'quiet', 
                        'config=', 'full', 'managed', 'managed-only',
                        'logfile=', 'owner=', 'mode=', 
-                       'output', 'debug', 'stack', ]
+                       'output', 'debug', 'stack', 'diagnostics', ]
 
 
 #######################################################################
@@ -208,6 +208,9 @@ def cli():
    if options.version:
       _version()
       return 0
+   if options.diagnostics:
+      _diagnostics()
+      return 0
 
    try:
       logfile = setupLogging(options)
@@ -218,6 +221,7 @@ def cli():
    logger.info("Cedar Backup run started.")
    logger.info("Options were [%s]" % options)
    logger.info("Logfile is [%s]" % logfile)
+   Diagnostics().logDiagnostics(method=logger.info)
 
    if options.config is None:
       logger.debug("Using default configuration file.")
@@ -296,8 +300,6 @@ class _ActionItem(object):
    type (_ActionItem before _ManagedActionItem) and then by index within type.
 
    @cvar SORT_ORDER: Defines a sort order to order properly between types.
-
-   @sort: __init__, index, module, function 
    """
    
    SORT_ORDER = 0
@@ -921,6 +923,7 @@ def _usage(fd=sys.stderr):
    fd.write("   -O, --output       Record some sub-command (i.e. cdrecord) output to the log\n")
    fd.write("   -d, --debug        Write debugging information to the log (implies --output)\n")
    fd.write("   -s, --stack        Dump a Python stack trace instead of swallowing exceptions\n") # exactly 80 characters in width!
+   fd.write("   -D, --diagnostics  Print runtime diagnostics to the screen and exit\n")
    fd.write("\n")
    fd.write(" The following actions may be specified:\n")
    fd.write("\n")
@@ -967,6 +970,23 @@ def _version(fd=sys.stdout):
    fd.write("\n")
 
 
+##########################
+# _diagnostics() function
+##########################
+
+def _diagnostics(fd=sys.stdout):
+   """
+   Prints runtime diagnostics information.
+   @param fd: File descriptor used to print information.
+   @note: The C{fd} is used rather than C{print} to facilitate unit testing.
+   """
+   fd.write("\n")
+   fd.write("Diagnostics:\n")
+   fd.write("\n")
+   Diagnostics().printDiagnostics(fd=fd, prefix="   ")
+   fd.write("\n")
+
+
 ##########################
 # setupLogging() function
 ##########################
@@ -1263,6 +1283,7 @@ class Options(object):
       self._output = False
       self._debug = False
       self._stacktrace = False
+      self._diagnostics = False
       self._actions = None
       self.actions = []    # initialize to an empty list; remainder are OK
       if argumentList is not None and argumentString is not None:
@@ -1375,6 +1396,11 @@ class Options(object):
             return -1
          else:
             return 1
+      if self._diagnostics != other._diagnostics:
+         if self._diagnostics < other._diagnostics:
+            return -1
+         else:
+            return 1
       if self._actions != other._actions:
          if self._actions < other._actions:
             return -1
@@ -1628,6 +1654,22 @@ class Options(object):
       """
       return self._stacktrace
 
+   def _setDiagnostics(self, value):
+      """
+      Property target used to set the diagnostics flag.
+      No validations, but we normalize the value to C{True} or C{False}.
+      """
+      if value:
+         self._diagnostics = True
+      else:
+         self._diagnostics = False
+
+   def _getDiagnostics(self):
+      """
+      Property target used to get the diagnostics flag.
+      """
+      return self._diagnostics
+
    def _setActions(self, value):
       """
       Property target used to set the actions list.
@@ -1665,6 +1707,7 @@ class Options(object):
    output = property(_getOutput, _setOutput, None, "Command-line output (C{-O,--output}) flag.")
    debug = property(_getDebug, _setDebug, None, "Command-line debug (C{-d,--debug}) flag.")
    stacktrace = property(_getStacktrace, _setStacktrace, None, "Command-line stacktrace (C{-s,--stack}) flag.")
+   diagnostics = property(_getDiagnostics, _setDiagnostics, None, "Command-line diagnostics (C{-D,--diagnostics}) flag.")
    actions = property(_getActions, _setActions, None, "Command-line actions list.")
 
 
@@ -1686,7 +1729,7 @@ class Options(object):
 
       @raise ValueError: If one of the validations fails.
       """
-      if not self.help and not self.version:
+      if not self.help and not self.version and not self.diagnostics:
          if self.actions is None or len(self.actions) == 0:
             raise ValueError("At least one action must be specified.")
       if self.managed and self.managedOnly:
@@ -1754,6 +1797,8 @@ class Options(object):
          argumentList.append("--debug")
       if self.stacktrace:
          argumentList.append("--stack")
+      if self.diagnostics:
+         argumentList.append("--diagnostics")
       if self.actions is not None:
          for action in self.actions:
             argumentList.append(action)
@@ -1817,6 +1862,8 @@ class Options(object):
          argumentString += "--debug "
       if self.stacktrace:
          argumentString += "--stack "
+      if self.diagnostics:
+         argumentString += "--diagnostics "
       if self.actions is not None:
          for action in self.actions:
             argumentString +=  "\"%s\" " % action
@@ -1883,6 +1930,8 @@ class Options(object):
          self.debug = True
       if switches.has_key("-s") or switches.has_key("--stack"):
          self.stacktrace = True
+      if switches.has_key("-D") or switches.has_key("--diagnostics"):
+         self.diagnostics = True
 
 
 #########################################################################
